#!/bin/zsh -f

setopt EXTENDED_GLOB NO_NOMATCH NUMERIC_GLOB_SORT BRACE_CCL RC_QUOTES
#unsetopt

############################ Generating patches ############################

LF=$(echo)

### Using patch ###
applypatch () {
  case "$1" in
    *.gz|*.Z|*.z) zcat $1 ;;
    *) cat $1 ;;
  esac | patch -E -p0 2>&1 | tee zap
  echo
  echo 'All results of the patch process were put in file "zap".'
}

getversion () {
  . version
  echo $MAJOR_VERSION.$MINOR_VERSION.$PATCH_LEVEL.$INTERNAL_REVISION
}

compare_versions () {
  if cmp --silent ${TUNES}/version ${OLDTUNES}/version
  then
    echo 'Old and new distributions have same version number !'
    exit 1
  else
    return 0  # versions are different as expected.
  fi
}

mkpatch () {
  OLDVER=$(cd $OLDTUNES ; getversion)
  PATCH=${PATCH_PREFIX}${OLDVER}-${TUNESVER}

  if [ -f $PATCH ] ; then ${=RM} $PATCH ; fi
  PATCH

  comprpatch
}

comprpatch () {
  #OLDVER=$(cd $OLDTUNES ; getversion)
  #PATCH=${PATCH_PREFIX}${OLDVER}-${TUNESVER}
  ls -sl ${PATCH}
  ${=COMPR} ${PATCH}
  ls -sl ${PATCH}${=COMPRSUFFIX}
}

mkdiff () {
  if [ -f "$1" ] ; then
    if [ -f "$2" ] ; then
      diff --unified "$1" "$2"
      # to save a little space, use -C0 instead of --unified
    else
      echo "Only in $(dirname $1): $(basename $1)"
      #touch -d '00:00:00GMT 01 Jan 1970' $2
      #diff --unified "$1" "$2"
      #/bin/rm $2
    fi
  else
    if [ -f "$2" ] ; then
      #echo "Only in $(dirname $2): $(basename $2)"
      touch -d '00:00:00GMT 01 Jan 1970' $1
      diff --unified "$1" "$2"
      /bin/rm $1
    else
      echo "File not found $2" >&2
    fi
  fi
}

function PATCH () {(
  #setopt -x
  PATCHDIR ""
  PATCHDIR src/
  PATCHDIR doc/

 ### add as many -x as necessary to exclude all unneeded files
 ### that may be generated or hand-written (temporary files, then)
#  mkdiff --recursive -X doc/archive -x '*-list' \
#    -x '.tmp*' -x 'patch.*' -x 'version.h' \
#    -x '*.[oasilbxpZ]' \
#    -x '*.obj' -x '*.bin' -x '*.exe' -x '*.co*' \
#    -x '*.?rf' -x '*.lst' -x '*.map' \
#    -x '*.tmp' -x '*.$$$' \
#    -x '*~' -x '#*#' \
#    -x '*.bak' -x '*.orig' -x '*.rej' \
#    -x 'z[aeiou]p' -x foo -x bar -x test -x 'core*' \
#    ${OLDTUNES} ${TUNES} >> ${PATCH}
#  A=$? ; if [ $A = 1 ] ; then return 0 ; else return $A  ; fi
)}

function PATCHDIR () {(
  : PATCHDIR
  FILES=$(LOGfiles $TUNES $1)
  : FILES $FILES
  OFILES=$(LOGfiles $OLDTUNES $1)
  : OFILES $OFILES
  AFILES=$(echo $FILES $LF $OFILES | sort -u)
  : AFILES $AFILES
  for i in ${=AFILES} ; do
    mkdiff $OLDTUNES/$i $TUNES/$i >> $PATCH
  done
)}

clean () {
   BAD="`find . '(' -name '*.orig' \
		-o -name '*.rej' \
		-o -name '*.bak' \
		-o -name '.tmp*' \
		-o -name 'patch.*' \
		-o -name 'version.h' \
		-o -name '*.[oasilbxpZ]' \
		-o -name '*.obj' \
		-o -name '*.bin' \
		-o -name '*.exe' \
		-o -name '*.co*' \
		-o -name '*.?rf' \
		-o -name '*.lst' \
		-o -name '*.map' \
		-o -name '*.tmp' \
		-o -name '*.$$$' \
		-o -name '*~' \
		-o -name '#*#' \
		-o -name 'z[aeiou]p' \
		-o -name foo \
		-o -name bar \
		-o -name test \
		-o -name 'core*' \
		')' \
		-print`"
   echo $BAD >&2
   /bin/rm -f ${=BAD}
}


################### LOGGING FUNCTIONS (FOR INCREMENTAL SAVE) #############
########################### using zip/unzip ##############################
function syssvdir () { echo ${SYSSVDIR:-$TUNES} }

function SVTDOC () {( LOGFILE=tunesdoc LOGDIR=$TUNES/doc LOGOUT )}
function SVTBIN () {( LOGFILE=tunesbin LOGDIR=$TUNES/    LOGOUT )}
function SVTSRC () {( LOGFILE=tunessrc LOGDIR=$TUNES/src LOGOUT )}
function SVTUNES () {( SVTDOC ; SVTBIN ; SVTSRC )}

function LOGfiles () {(
  setopt NULL_GLOB
  LOGDIR=$1
  LOGSUBDIR=$2
  : LOGfiles $LOGDIR $LOGSUBDIR

  cd $LOGDIR &&
  {
    FILES=$(cd ./$LOGSUBDIR ;
	   eval echo $(cat .Logrc | grep -v '^[#lrA-Z]' |
 	   ( while read c f x ; do echo $f ; done )))
    for i in ${=FILES} ; do
      if [ -d ${LOGSUBDIR}$i ]
      then
        I=${LOGSUBDIR}$i/**/*
        for j in ${=I}
        do if [ -d $j ] ; then echo $j/ ; else echo $j ; fi
        done
      else echo ${LOGSUBDIR}$i
      fi
    done
  }
)}

function LOGOUT () {(
  setopt NULL_GLOB
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  LOGLOG=/tmp/Log-$HOST--$LOGFILE.log
  LOGSUBDIR=$3 ; if [ -z "$3" ] ; then echo -n > $LOGLOG ; fi
  SVDIR=$(syssvdir)

  cd $LOGDIR &&
  { echo " Saving important files in $LOGDIR/$LOGSUBDIR:"

    FILES=$(cd ./$LOGSUBDIR ;
            eval echo $(cat .Logrc | grep -v '^[#lrA-Z]' |
 	   ( while read c f x ; do echo $f ; done )))
    for i in ${=FILES} ; do
      if [ -d ${LOGSUBDIR}$i ]
      then
        I=${LOGSUBDIR}$i/**/*
        for j in ${=I} 
        do if [ -d $j ] ; then echo $j/ ; else echo $j ; fi
        done
      else echo $i
      fi
    done  >> $LOGLOG
    zip -uy9r /$SVDIR/$LOGFILE ${=FILES}

    SUBDIRS=$(eval echo $(cat ${LOGSUBDIR}.Logrc | grep '^r' |
  	      cut -d'	' -f1 | cut -d' ' -f2))
    for i in ${=SUBDIRS} ; do LOGOUT $LOGFILE $LOGDIR ${LOGSUBDIR}${i}/ ; done
  }
)}

function LOGIN () {(
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  cd $LOGDIR && unzip -uo /$(syssvdir)/$LOGFILE
)}

function CLOG () {( # clean logfile
  setopt NULL_GLOB
  if [ -n "$1" ] ; then LOGFILE=$1 ;
    if [ -n "$2" ] ; then LOGDIR=$2 ; fi ; fi
  SVDIR=$(syssvdir)
  LOGLOG=/tmp/Log-$HOST--$LOGFILE.log

  GOODFILES=$(cat $LOGLOG)
  : GF $GOODFILES
  ZF=$(unzip -l /$SVDIR/$LOGFILE | cut -c28-)
  : ZF $ZF
  ZF=(${=ZF})
  : ZF $ZF
  ZF=$ZF[7,-3]
  : ZF $ZF
  ZIPPEDFILES=${=ZF}

  : ZIPPEDFILES=$ZIPPEDFILES
  : ZIPPY $ZIPPEDFILES[10,20]
  : GOODFILES=$GOODFILES
  : GOODY $GOODFILES[10,20]

  BADFILES=$(Set_Difference "$ZIPPEDFILES" "$GOODFILES")
  : BF $BADFILES
  echo zip -d /$SVDIR/$LOGFILE $BADFILES

  ### doesn't work: LOGOUT uses zip -r, so that all files below
  ### a zip -r 'ed directory will be considered as BAD !!!
)}



function Is_Element () {(
  for i in ${=2} ; do if [ "$1" = "$i" ]; then return 0 ; fi ; done
  return 1
)}

function Set_Difference () {
# works for sets as IFS-separated list, not as arrays
  for i in ${=1} ; do if ! Is_Element $i $2 ; then echo $i ; fi ; done
}


########################## interactive use only #############################

function srcfiles () {
  echo info/{i386*,BIOS-API}.nfo src/LLL/**/*.{S,m4}
}

function t () {

}

#function main () {
  "$@"
#}
